@@ -2,6 +2,7 @@
  * Copyright (C) 2003 Sun Microsystems, Inc.
  * Copyright (C) 2004 Red Hat, Inc.
  * Copyright (C) 2004 Novell, Inc.
+ * Copyright © 2010 Codethink Limited
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +22,7 @@
  * Authors:
  *      Mark McLoughlin <mark@skynet.ie>
  *      Federico Mena Quintero <federico@ximian.com>
+ *      Ryan Lortie <desrt@desrt.ca>
  *
  *
  *   The screen polling code is based on XUpdateScanner from
@@ -74,12 +76,12 @@ struct _VinoFBPrivate
   XImage          *tile;
   XShmSegmentInfo  tile_x_shm_info;
 
-  GdkRegion       *damage_region;
+  cairo_region_t  *damage_region;
 
   guint            update_timeout;
 
 #ifdef HAVE_XDAMAGE
-  GdkRegion       *pending_damage;
+  cairo_region_t  *pending_damage;
   guint            damage_idle_handler;
 
   Damage           xdamage;
@@ -142,7 +144,7 @@ emit_size_changed (VinoFB *vfb)
 
 static gboolean
 vino_fb_get_image (VinoFB          *vfb,
-		   GdkDrawable     *drawable,
+		   GdkWindow       *window,
 		   XImage          *image,
 		   XShmSegmentInfo *x_shm_info,
 		   gboolean         is_x_shm_segment,
@@ -154,13 +156,13 @@ vino_fb_get_image (VinoFB          *vfb,
   Drawable xdrawable;
   int      error;
 
-  g_assert (vfb != NULL && drawable != NULL && image != NULL && x_shm_info != NULL);
+  g_assert (vfb != NULL && window != NULL && image != NULL && x_shm_info != NULL);
 
-  xdrawable = GDK_DRAWABLE_XID (drawable);
+  xdrawable = GDK_WINDOW_XID (window);
 
   gdk_error_trap_push ();
 
-#ifdef HAVE_XSHM  
+#ifdef HAVE_XSHM
   if (is_x_shm_segment && image->width == width && image->height == height)
     {
       XShmGetImage (vfb->priv->xdisplay,
@@ -181,7 +183,7 @@ vino_fb_get_image (VinoFB          *vfb,
 
   if ((error = gdk_error_trap_pop ()))
     {
-#ifdef G_ENABLE_DEBUG
+#ifdef GNOME_ENABLE_DEBUG
       char error_text [64];
 
       XGetErrorText (vfb->priv->xdisplay, error, error_text, 63);
@@ -312,8 +314,8 @@ vino_fb_create_image (VinoFB           *vfb,
 }
 
 static gboolean
-vino_fb_copy_tile (VinoFB       *vfb,
-		   GdkRectangle *rect)
+vino_fb_copy_tile (VinoFB                      *vfb,
+		   const cairo_rectangle_int_t *rect)
 {
   XImage *fb_image;
   char   *src;
@@ -390,7 +392,7 @@ vino_fb_poll_scanline (VinoFB *vfb,
 
       if (memcmp (dest, src, width * bytes_per_pixel) != 0)
 	{
-	  GdkRectangle rect;
+	  cairo_rectangle_int_t rect;
 
 	  rect.x = x;
 	  rect.y = line - (line % TILE_HEIGHT);
@@ -402,9 +404,9 @@ vino_fb_poll_scanline (VinoFB *vfb,
 	  if (vino_fb_copy_tile (vfb, &rect))
 	    {
 	      if (!vfb->priv->damage_region)
-		vfb->priv->damage_region = gdk_region_rectangle (&rect);
+		vfb->priv->damage_region = cairo_region_create_rectangle (&rect);
 	      else
-		gdk_region_union_with_rect (vfb->priv->damage_region, &rect);
+	        cairo_region_union_rectangle (vfb->priv->damage_region, &rect);
 	    }
 
 	  retval = TRUE;
@@ -444,7 +446,7 @@ vino_fb_poll_screen (VinoFB *vfb)
 	screen_damaged = TRUE;
     }
   
-  vfb->priv->n_scanline = ++vfb->priv->n_scanline % N_SCANLINES;
+  vfb->priv->n_scanline = (vfb->priv->n_scanline + 1) % N_SCANLINES;
 
   if (!already_damaged && screen_damaged)
     emit_damage_notify (vfb);
@@ -461,7 +463,7 @@ vino_fb_finalize_xdamage (VinoFB *vfb)
   vfb->priv->damage_idle_handler = 0;
 
   if (vfb->priv->pending_damage)
-    gdk_region_destroy (vfb->priv->pending_damage);
+    cairo_region_destroy (vfb->priv->pending_damage);
   vfb->priv->pending_damage = NULL;
 
   if (vfb->priv->fb_pixmap)
@@ -515,7 +517,7 @@ static void
 vino_fb_finalize_screen_data (VinoFB *vfb)
 {
   if (vfb->priv->damage_region)
-    gdk_region_destroy (vfb->priv->damage_region);
+    cairo_region_destroy (vfb->priv->damage_region);
   vfb->priv->damage_region = NULL;
 
   if (vfb->priv->use_xdamage)
@@ -554,19 +556,19 @@ vino_fb_screen_size_changed (VinoFB    *vfb,
 static gboolean
 vino_fb_xdamage_idle_handler (VinoFB *vfb)
 {
-  GdkRectangle *damage = NULL;
-  XRectangle    xdamage;
-  int           n_rects;
-  int           error;
 
-  g_assert (!gdk_region_empty (vfb->priv->pending_damage));
+  cairo_rectangle_int_t damage;
+  XRectangle            xdamage;
+  int                   error;
 
-  gdk_region_get_rectangles (vfb->priv->pending_damage, &damage, &n_rects);
+  g_assert (!cairo_region_is_empty (vfb->priv->pending_damage));
 
-  xdamage.x      = damage->x;
-  xdamage.y      = damage->y;
-  xdamage.width  = damage->width;
-  xdamage.height = damage->height;
+  cairo_region_get_rectangle (vfb->priv->pending_damage, 0, &damage);
+
+  xdamage.x      = damage.x;
+  xdamage.y      = damage.y;
+  xdamage.width  = damage.width;
+  xdamage.height = damage.height;
 
   dprintf (POLLING, "Updating damaged region in idle: %d %d %dx%d\n",
 	   damage->x, damage->y, damage->width, damage->height);
@@ -584,35 +586,35 @@ vino_fb_xdamage_idle_handler (VinoFB *vfb)
   if (vfb->priv->use_x_shm)
     {
       XCopyArea (vfb->priv->xdisplay,
-		 GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+		 GDK_WINDOW_XID (vfb->priv->root_window),
 		 vfb->priv->fb_pixmap,
 		 vfb->priv->xdamage_copy_gc,
-		 damage->x,
-		 damage->y,
-		 damage->width,
-		 damage->height,
-		 damage->x,
-		 damage->y);
+		 damage.x,
+		 damage.y,
+		 damage.width,
+		 damage.height,
+		 damage.x,
+		 damage.y);
       XSync (vfb->priv->xdisplay, False);
     }
   else
     {
       XGetSubImage (vfb->priv->xdisplay,
-		    GDK_WINDOW_XWINDOW (vfb->priv->root_window),
-		    damage->x,
-		    damage->y,
-		    damage->width,
-		    damage->height,
+		    GDK_WINDOW_XID (vfb->priv->root_window),
+		    damage.x,
+		    damage.y,
+		    damage.width,
+		    damage.height,
 		    AllPlanes,
 		    ZPixmap,
 		    vfb->priv->fb_image,
-		    damage->x,
-		    damage->y);
+		    damage.x,
+		    damage.y);
     }
 
   if ((error = gdk_error_trap_pop ()))
     {
-#ifdef G_ENABLE_DEBUG
+#ifdef GNOME_ENABLE_DEBUG
       char error_text [64];
 
       XGetErrorText (vfb->priv->xdisplay, error, error_text, 63);
@@ -632,24 +634,22 @@ vino_fb_xdamage_idle_handler (VinoFB *vfb)
 
   /* add damage to our region */
   if (vfb->priv->damage_region)
-    gdk_region_union_with_rect (vfb->priv->damage_region, damage);
+    cairo_region_union_rectangle (vfb->priv->damage_region, &damage);
   else
-    vfb->priv->damage_region = gdk_region_rectangle (damage);
+    vfb->priv->damage_region = cairo_region_create_rectangle (&damage);
 
   emit_damage_notify (vfb);
 
  out:
   {
-    GdkRegion *tmp;
+    cairo_region_t *tmp;
 
-    tmp = gdk_region_rectangle (damage);
-    gdk_region_subtract (vfb->priv->pending_damage, tmp);
-    gdk_region_destroy (tmp);
+    tmp = cairo_region_create_rectangle (&damage);
+    cairo_region_subtract (vfb->priv->pending_damage, tmp);
+    cairo_region_destroy (tmp);
   }
 
-  g_free (damage);
-
-  if (gdk_region_empty (vfb->priv->pending_damage))
+  if (cairo_region_is_empty (vfb->priv->pending_damage))
     {
       vfb->priv->damage_idle_handler = 0;
       return FALSE;
@@ -663,9 +663,9 @@ vino_fb_xdamage_event_filter (GdkXEvent *xevent,
 			      GdkEvent  *event,
 			      VinoFB    *vfb)
 {
-  XEvent             *xev = (XEvent *) xevent;
-  XDamageNotifyEvent *notify;
-  GdkRectangle        damage;
+  XEvent                *xev = (XEvent *) xevent;
+  XDamageNotifyEvent    *notify;
+  cairo_rectangle_int_t  damage;
 
   if (xev->type != vfb->priv->xdamage_notify_event)
     return GDK_FILTER_CONTINUE;
@@ -681,7 +681,7 @@ vino_fb_xdamage_event_filter (GdkXEvent *xevent,
 	   damage.x, damage.y, damage.width, damage.height,
 	   notify->more ? "(true)" : "(false)", notify->level);
 
-  gdk_region_union_with_rect (vfb->priv->pending_damage, &damage);
+  cairo_region_union_rectangle (vfb->priv->pending_damage, &damage);
 
   if (!vfb->priv->damage_idle_handler)
     vfb->priv->damage_idle_handler =
@@ -714,7 +714,7 @@ vino_fb_init_xdamage (VinoFB *vfb)
   vfb->priv->xdamage_notify_event = event_base + XDamageNotify;
 
   vfb->priv->xdamage = XDamageCreate (vfb->priv->xdisplay,
-				      GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+				      GDK_WINDOW_XID (vfb->priv->root_window),
 				      XDamageReportDeltaRectangles);
   if (vfb->priv->xdamage == None)
     return;
@@ -729,7 +729,7 @@ vino_fb_init_xdamage (VinoFB *vfb)
 
   values.subwindow_mode = IncludeInferiors;
   vfb->priv->xdamage_copy_gc = XCreateGC (vfb->priv->xdisplay,
-					  GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+					  GDK_WINDOW_XID (vfb->priv->root_window),
 					  GCSubwindowMode,
 					  &values);
 
@@ -740,7 +740,7 @@ vino_fb_init_xdamage (VinoFB *vfb)
 			 (GdkFilterFunc) vino_fb_xdamage_event_filter,
 			 vfb);
 
-  vfb->priv->pending_damage = gdk_region_new ();
+  vfb->priv->pending_damage = cairo_region_create ();
 
   vfb->priv->use_xdamage = TRUE;
 #endif
@@ -818,7 +818,7 @@ vino_fb_init_fb_image (VinoFB *vfb)
       if (vfb->priv->use_x_shm)
 	{
 	  vfb->priv->fb_pixmap = XShmCreatePixmap (vfb->priv->xdisplay,
-						   GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+						   GDK_WINDOW_XID (vfb->priv->root_window),
 						   vfb->priv->fb_image->data,
 						   &vfb->priv->fb_image_x_shm_info,
 						   vfb->priv->fb_image->width,
@@ -842,7 +842,7 @@ vino_fb_init_fb_image (VinoFB *vfb)
     {
       vfb->priv->fb_image =
 	XGetImage (vfb->priv->xdisplay,
-		   GDK_WINDOW_XWINDOW (vfb->priv->root_window),
+		   GDK_WINDOW_XID (vfb->priv->root_window),
 		   0, 0,
 		   gdk_screen_get_width  (vfb->priv->screen),
 		   gdk_screen_get_height (vfb->priv->screen),
@@ -1176,65 +1176,53 @@ vino_fb_get_color_masks (VinoFB *vfb,
 }
 
 static inline void
-vino_fb_debug_dump_damage (VinoFB       *vfb,
-			   GdkRectangle *rects,
-			   int           n_rects)
+vino_fb_debug_dump_damage (VinoFB         *vfb,
+			   cairo_region_t *region)
 {
-#ifdef G_ENABLE_DEBUG
+#ifdef GNOME_ENABLE_DEBUG
   if (_vino_debug_flags & VINO_DEBUG_POLLING)
     {
-      GdkRectangle clipbox;
-      int          area;
-      int          i;
+      cairo_rectangle_int_t extents;
+      int                   n_rects;
+      int                   area;
+      int                   i;
 
-      gdk_region_get_clipbox (vfb->priv->damage_region, &clipbox);
+      cairo_region_get_extents (vfb->priv->damage_region, &extents);
 
-      fprintf (stderr, "Dump of damage region: clipbox (%d, %d) (%d x %d)\n",
-	       clipbox.x, clipbox.y, clipbox.width, clipbox.height);
+      fprintf (stderr, "Dump of damage region: extents (%d, %d) (%d x %d)\n",
+               extents.x, extents.y, extents.width, extents.height);
 
       area = 0;
+      n_rects = cairo_region_num_rectangles (region);
       for (i = 0; i < n_rects; i++)
-	{
-	  fprintf (stderr, "\t(%d, %d) (%d x %d)\n",
-		   rects [i].x, rects [i].y, rects [i].width, rects [i].height);
-	  area += rects [i].width * rects [i].height;
-	}
+        {
+          cairo_rectangle_int_t rect;
+
+          cairo_region_get_rectangle (region, i, &rect);
+
+          fprintf (stderr, "\t(%d, %d) (%d x %d)\n",
+                   rect.x, rect.y, rect.width, rect.height);
+          area += rect.width * rect.height;
+        }
 
       fprintf (stderr, "Bounding area %d, damaged area %d ... (%d%%)\n",
-	       clipbox.width * clipbox.height, area,
-	       (area * 100) / (clipbox.width * clipbox.height));
+               extents.width * extents.height, area,
+               (area * 100) / (extents.width * extents.height));
     }
 #endif
 }
 
-GdkRectangle *
-vino_fb_get_damage (VinoFB   *vfb,
-		    int      *n_rects,
-		    gboolean  clear_damage)
-
+cairo_region_t *
+vino_fb_steal_damage (VinoFB *vfb)
 {
-  GdkRectangle *retval;
+  cairo_region_t *retval;
 
   g_return_val_if_fail (VINO_IS_FB (vfb), NULL);
-  g_return_val_if_fail (n_rects != NULL, NULL);
 
-  if (!vfb->priv->damage_region)
-    {
-      *n_rects = 0;
-      return NULL;
-    }
+  retval = vfb->priv->damage_region;
+  vino_fb_debug_dump_damage (vfb, retval);
 
-  retval = NULL;
-  *n_rects = 0;
-  gdk_region_get_rectangles (vfb->priv->damage_region, &retval, n_rects);
-
-  vino_fb_debug_dump_damage (vfb, retval, *n_rects);
- 
-  if (clear_damage)
-    {
-      gdk_region_destroy (vfb->priv->damage_region);
-      vfb->priv->damage_region = NULL;
-    }
+  vfb->priv->damage_region = NULL;
 
   return retval;
 }
