@@ -24,6 +24,7 @@
 
 #include <config.h>
 #include <gtk/gtk.h>
+#include <gio/gdesktopappinfo.h>
 #include <string.h>
 #ifdef VINO_ENABLE_LIBNOTIFY
 #include <libnotify/notify.h>
@@ -117,7 +118,7 @@ vino_status_icon_update_state (VinoStatusIcon *icon)
   else
     visible = visible && (icon->priv->visibility == VINO_STATUS_ICON_VISIBILITY_ALWAYS);
 
-  gtk_status_icon_set_tooltip (GTK_STATUS_ICON (icon), tooltip);
+  gtk_status_icon_set_tooltip_text (GTK_STATUS_ICON (icon), tooltip);
   gtk_status_icon_set_visible (GTK_STATUS_ICON (icon), visible);
 
   g_free (tooltip);
@@ -199,16 +200,23 @@ static void
 vino_status_icon_preferences (VinoStatusIcon *icon)
 {
   GdkScreen *screen;
+  GDesktopAppInfo *info;
+  GdkAppLaunchContext *context;
   GError *error = NULL;
 
   screen = gtk_status_icon_get_screen (GTK_STATUS_ICON (icon));
-  if (!gdk_spawn_command_line_on_screen (screen, "vino-preferences", &error))
+  info = g_desktop_app_info_new ("vino-preferences.desktop");
+  context = gdk_display_get_app_launch_context (gdk_screen_get_display (screen));
+  if (!g_app_info_launch (G_APP_INFO (info), NULL, G_APP_LAUNCH_CONTEXT (context), &error))
     {
       vino_util_show_error (_("Error displaying preferences"),
-			    error->message,
-			    NULL);
+                            error->message,
+                            NULL);
       g_error_free (error);
     }
+
+  g_object_unref (info);
+  g_object_unref (context);
 }
 
 static void
@@ -335,15 +343,19 @@ vino_status_icon_disconnect_confirm (VinoStatusIconNotify *a)
   if (client != NULL)
     {
       /* Translators: %s is a hostname */
-      primary_msg   = g_strdup_printf (_("Are you sure you want to disconnect '%s'?"),
-                                       vino_client_get_hostname (client));
-      secondary_msg = g_strdup_printf (_("The remote user from '%s' will be disconnected. Are you sure?"),
-                                       vino_client_get_hostname (client));
+      primary_msg   = g_strdup_printf
+          (_("Are you sure you want to disconnect '%s'?"),
+          vino_client_get_hostname (client));
+      secondary_msg = g_strdup_printf
+          (_("The remote user from '%s' will be disconnected. Are you sure?"),
+          vino_client_get_hostname (client));
     }
   else
     {
-      primary_msg   = g_strdup (_("Are you sure you want to disconnect all clients?"));
-      secondary_msg = g_strdup (_("All remote users will be disconnected. Are you sure?"));
+      primary_msg   = g_strdup
+          (_("Are you sure you want to disconnect all clients?"));
+      secondary_msg = g_strdup
+          (_("All remote users will be disconnected. Are you sure?"));
     }
 
   icon->priv->disconnect_dialog = gtk_message_dialog_new (NULL,
@@ -420,7 +432,8 @@ vino_status_icon_popup_menu (GtkStatusIcon *status_icon,
       a->client = client;
 
       /* Translators: %s is a hostname */
-      str = g_strdup_printf (_("Disconnect %s"), vino_client_get_hostname (client));
+      str = g_strdup_printf (_("Disconnect %s"),
+          vino_client_get_hostname (client));
 
       item  = gtk_image_menu_item_new_with_label (str);
       gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
@@ -601,22 +614,32 @@ vino_status_icon_show_new_client_notification (gpointer user_data)
 
   if (vino_server_get_view_only (icon->priv->server))
     {
+      /* Translators: %s is a hostname */
       summary = _("Another user is viewing your desktop");
-      body = g_strdup_printf (_("A user on the computer '%s' is remotely viewing your desktop."),
-                              vino_client_get_hostname (client));
+      body = g_strdup_printf
+          (_("A user on the computer '%s' is remotely viewing your desktop."),
+          vino_client_get_hostname (client));
     }
   else
     {
+      /* Translators: %s is a hostname */
       summary = _("Another user is controlling your desktop");
-      body = g_strdup_printf (_("A user on the computer '%s' is remotely controlling your desktop."),
-                              vino_client_get_hostname (client));
+      body = g_strdup_printf
+          (_("A user on the computer '%s' is remotely controlling "
+          "your desktop."), vino_client_get_hostname (client));
     }
 
   icon->priv->new_client_notification =
+#ifdef VINO_HAS_LIBNOTIFY_0_7
+    notify_notification_new (summary,
+                             body,
+                             "preferences-desktop-remote-desktop");
+#else
     notify_notification_new_with_status_icon (summary,
                                               body,
                                               "preferences-desktop-remote-desktop",
                                               GTK_STATUS_ICON (icon));
+#endif
 
   g_free (body);
 
